@@ -1,3 +1,5 @@
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
+
 /* Metacity preferences */
 
 /* 
@@ -1379,10 +1381,42 @@ button_function_from_string (const char *str)
     return META_BUTTON_FUNCTION_MAXIMIZE;
   else if (strcmp (str, "close") == 0)
     return META_BUTTON_FUNCTION_CLOSE;
-  else
+  else if (strcmp (str, "shade") == 0)
+    return META_BUTTON_FUNCTION_SHADE;
+  else if (strcmp (str, "above") == 0)
+    return META_BUTTON_FUNCTION_ABOVE;
+  else if (strcmp (str, "stick") == 0)
+    return META_BUTTON_FUNCTION_STICK;
+  else 
+    /* don't know; give up */
     return META_BUTTON_FUNCTION_LAST;
 }
 
+static MetaButtonFunction
+button_opposite_function (MetaButtonFunction ofwhat)
+{
+  switch (ofwhat)
+    {
+    case META_BUTTON_FUNCTION_SHADE:
+      return META_BUTTON_FUNCTION_UNSHADE;
+    case META_BUTTON_FUNCTION_UNSHADE:
+      return META_BUTTON_FUNCTION_SHADE;
+
+    case META_BUTTON_FUNCTION_ABOVE:
+      return META_BUTTON_FUNCTION_UNABOVE;
+    case META_BUTTON_FUNCTION_UNABOVE:
+      return META_BUTTON_FUNCTION_ABOVE;
+
+    case META_BUTTON_FUNCTION_STICK:
+      return META_BUTTON_FUNCTION_UNSTICK;
+    case META_BUTTON_FUNCTION_UNSTICK:
+      return META_BUTTON_FUNCTION_STICK;
+
+    default:
+      return META_BUTTON_FUNCTION_LAST;
+    }
+}
+
 static gboolean
 update_button_layout (const char *value)
 {
@@ -1433,6 +1467,11 @@ update_button_layout (const char *value)
               new_layout.left_buttons[i] = f;
               used[f] = TRUE;
               ++i;
+
+              f = button_opposite_function (f);
+
+              if (f != META_BUTTON_FUNCTION_LAST)
+                new_layout.left_buttons[i++] = f;
             }
           else
             {
@@ -1471,7 +1510,12 @@ update_button_layout (const char *value)
               new_layout.right_buttons[i] = f;
               used[f] = TRUE;
               ++i;
-            }
+
+              f = button_opposite_function (f);
+
+              if (f != META_BUTTON_FUNCTION_LAST)
+                new_layout.right_buttons[i++] = f;
+             }
           else
             {
               meta_topic (META_DEBUG_PREFS, "Ignoring unknown or already-used button name \"%s\"\n",
@@ -1896,6 +1940,14 @@ static MetaKeyPref window_bindings[] = {
   { META_KEYBINDING_LOWER, NULL, FALSE },
   { META_KEYBINDING_MAXIMIZE_VERTICALLY, NULL, FALSE },
   { META_KEYBINDING_MAXIMIZE_HORIZONTALLY, NULL, FALSE },
+  { META_KEYBINDING_MOVE_TO_CORNER_NW, NULL, FALSE },
+  { META_KEYBINDING_MOVE_TO_CORNER_NE, NULL, FALSE },
+  { META_KEYBINDING_MOVE_TO_CORNER_SW, NULL, FALSE },
+  { META_KEYBINDING_MOVE_TO_CORNER_SE, NULL, FALSE },
+  { META_KEYBINDING_MOVE_TO_SIDE_N, NULL, FALSE },
+  { META_KEYBINDING_MOVE_TO_SIDE_S, NULL, FALSE },
+  { META_KEYBINDING_MOVE_TO_SIDE_E, NULL, FALSE },
+  { META_KEYBINDING_MOVE_TO_SIDE_W, NULL, FALSE },
   { NULL, NULL, FALSE }
 };
 
